namespace com.progsbase.no.inductive.libraries.Regression_v0_1_4{

using static System.Math;

public class Matrix{
	public MatrixRow [] r;
}
public class MatrixArrayReference{
	public Matrix [] matrices;
}
public class MatrixReference{
	public Matrix matrix;
}
public class MatrixRow{
	public double [] c;
}
public class BooleanArrayReference{
	public bool [] booleanArray;
}
public class BooleanReference{
	public bool booleanValue;
}
public class CharacterReference{
	public char characterValue;
}
public class NumberArrayReference{
	public double [] numberArray;
}
public class NumberReference{
	public double numberValue;
}
public class StringArrayReference{
	public StringReference [] stringArray;
}
public class StringReference{
	public char [] stringx;
}
public class Regression{
	public static Matrix OrdinaryLeastSquares(Matrix X, Matrix y){
		Matrix B;
		Matrix Xt, t1, t2, t3;

		/* Formula: B = (X^t * X)^-1 * X^t * y*/
		Xt = TransposeToNew(X);
		t1 = MultiplyToNew(Xt, X);
		t2 = CreateSquareMatrix(NumberOfRows(t1));
		Inverse(t1, t2);
		t3 = MultiplyToNew(t2, Xt);
		B = MultiplyToNew(t3, y);

		delete(Xt);
		delete(t1);
		delete(t2);
		delete(t3);

		return B;
	}


	public static Matrix OrdinaryLeastSquaresWithIntercept(Matrix X, Matrix y){
		Matrix B;
		Matrix Ones, XWithOnes;

		Ones = CreateMatrix(NumberOfRows(X), 1d);
		Fill(Ones, 1d);
		XWithOnes = ColumnCombineMatricesToNew(Ones, X);

		B = OrdinaryLeastSquares(XWithOnes, y);

		delete(XWithOnes);
		delete(Ones);

		return B;
	}


	public static void Add(Matrix a, Matrix b){
		double m, n;
		double d;

		d = NumberOfRows(a);
		for(m = 0d; m < d; m = m + 1d){
			for(n = 0d; n < d; n = n + 1d){
				a.r[(int)(m)].c[(int)(n)] = a.r[(int)(m)].c[(int)(n)] + b.r[(int)(m)].c[(int)(n)];
			}
		}
	}


	public static void Assign(Matrix A, Matrix B){
		double m, n;
		double r, c;

		r = NumberOfRows(A);
		c = NumberOfColumns(A);
		for(m = 0d; m < r; m = m + 1d){
			for(n = 0d; n < c; n = n + 1d){
				A.r[(int)(m)].c[(int)(n)] = B.r[(int)(m)].c[(int)(n)];
			}
		}
	}


	public static void Subtract(Matrix a, Matrix b){
		double m, n;
		double r, c;

		r = NumberOfRows(a);
		c = NumberOfColumns(a);
		for(m = 0d; m < r; m = m + 1d){
			for(n = 0d; n < c; n = n + 1d){
				a.r[(int)(m)].c[(int)(n)] = a.r[(int)(m)].c[(int)(n)] - b.r[(int)(m)].c[(int)(n)];
			}
		}
	}


	public static Matrix SubtractToNew(Matrix a, Matrix b){
		Matrix X;

		X = CreateCopyOfMatrix(a);
		Subtract(X, b);

		return X;
	}


	public static void ScalarMultiply(Matrix A, double b){
		double m, n;
		double r, c;

		r = NumberOfRows(A);
		c = NumberOfColumns(A);
		for(m = 0d; m < r; m = m + 1d){
			for(n = 0d; n < c; n = n + 1d){
				A.r[(int)(m)].c[(int)(n)] = b*A.r[(int)(m)].c[(int)(n)];
			}
		}
	}


	public static void ScalarDivide(Matrix A, double b){
		double m, n;
		double r, c;

		r = NumberOfRows(A);
		c = NumberOfColumns(A);
		for(m = 0d; m < r; m = m + 1d){
			for(n = 0d; n < c; n = n + 1d){
				A.r[(int)(m)].c[(int)(n)] = A.r[(int)(m)].c[(int)(n)]/b;
			}
		}
	}


	public static void ElementWisePower(Matrix A, double p){
		double m, n;
		double r, c;

		r = NumberOfRows(A);
		c = NumberOfColumns(A);

		for(m = 0d; m < r; m = m + 1d){
			for(n = 0d; n < c; n = n + 1d){
				A.r[(int)(m)].c[(int)(n)] = Pow(A.r[(int)(m)].c[(int)(n)], p);
			}
		}
	}


	public static Matrix ScalarMultiplyToNew(Matrix A, double b){
		Matrix matrix;

		matrix = CreateMatrix(NumberOfRows(A), NumberOfColumns(A));
		ScalarMultiply(matrix, b);

		return matrix;
	}


	public static Matrix MultiplyToNew(Matrix a, Matrix b){
		double rows, cols;
		Matrix x;

		rows = NumberOfRows(a);
		cols = NumberOfColumns(b);
		x = CreateMatrix(rows, cols);
		Multiply(x, a, b);

		return x;
	}


	public static void Multiply(Matrix x, Matrix a, Matrix b){
		double m, n;
		double rows, cols, d;
		double i, s;

		rows = NumberOfRows(a);
		cols = NumberOfColumns(b);
		d = NumberOfColumns(a);

		for(m = 0d; m < rows; m = m + 1d){
			for(n = 0d; n < cols; n = n + 1d){
				s = 0d;

				for(i = 0d; i < d; i = i + 1d){
					s = s + a.r[(int)(m)].c[(int)(i)]*b.r[(int)(i)].c[(int)(n)];
				}

				x.r[(int)(m)].c[(int)(n)] = s;
			}
		}
	}


	public static Matrix CreateSquareMatrix(double d){
		double m, n;
		Matrix matrix;

		matrix = new Matrix();
		matrix.r = new MatrixRow [(int)(d)];
		for(m = 0d; m < d; m = m + 1d){
			matrix.r[(int)(m)] = new MatrixRow();
			matrix.r[(int)(m)].c = new double [(int)(d)];
			for(n = 0d; n < d; n = n + 1d){
				matrix.r[(int)(m)].c[(int)(n)] = 0d;
			}
		}

		return matrix;
	}


	public static Matrix CreateMatrix(double rows, double cols){
		double m, n;
		Matrix matrix;

		matrix = new Matrix();
		matrix.r = new MatrixRow [(int)(rows)];
		for(m = 0d; m < rows; m = m + 1d){
			matrix.r[(int)(m)] = new MatrixRow();
			matrix.r[(int)(m)].c = new double [(int)(cols)];
			for(n = 0d; n < cols; n = n + 1d){
				matrix.r[(int)(m)].c[(int)(n)] = 0d;
			}
		}

		return matrix;
	}


	public static Matrix CreateIdentityMatrix(double d){
		double m;
		Matrix matrix;

		matrix = CreateSquareMatrix(d);
		Fill(matrix, 0d);

		for(m = 0d; m < d; m = m + 1d){
			matrix.r[(int)(m)].c[(int)(m)] = 1d;
		}

		return matrix;
	}


	public static void Transpose(Matrix a){
		double m, n;
		double rows, cols, tmp;

		cols = NumberOfRows(a);
		rows = NumberOfColumns(a);

		for(m = 0d; m < cols; m = m + 1d){
			for(n = 0d; n < rows; n = n + 1d){
				tmp = a.r[(int)(n)].c[(int)(m)];
				a.r[(int)(n)].c[(int)(m)] = a.r[(int)(m)].c[(int)(n)];
				a.r[(int)(m)].c[(int)(n)] = tmp;
			}
		}
	}


	public static Matrix TransposeToNew(Matrix a){
		double m, n;
		double rows, cols;
		Matrix c;

		cols = NumberOfRows(a);
		rows = NumberOfColumns(a);

		c = CreateMatrix(rows, cols);

		for(m = 0d; m < cols; m = m + 1d){
			for(n = 0d; n < rows; n = n + 1d){
				c.r[(int)(n)].c[(int)(m)] = a.r[(int)(m)].c[(int)(n)];
			}
		}

		return c;
	}


	public static void CofactorOfMatrix(Matrix mat, Matrix temp, double p, double q, double n){
		double i, j;
		double row, col;

		i = 0d;
		j = 0d;

		for(row = 0d; row < n; row = row + 1d){
			for(col = 0d; col < n; col = col + 1d){
				if(row != p && col != q){
					temp.r[(int)(i)].c[(int)(j)] = mat.r[(int)(row)].c[(int)(col)];
					j = j + 1d;

					if(j == n - 1d){
						j = 0d;
						i = i + 1d;
					}
				}
			}
		}
	}


	public static double DeterminantOfSubmatrix(Matrix mat, double n){
		double D, f, sign;
		Matrix temp;

		D = 0d;

		if(n == 1d){
			D = mat.r[0].c[0];
		}else{
			temp = CreateSquareMatrix(n);

			sign = 1d;

			for(f = 0d; f < n; f = f + 1d){
				CofactorOfMatrix(mat, temp, 0d, f, n);
				D = D + sign*mat.r[0].c[(int)(f)]*DeterminantOfSubmatrix(temp, n - 1d);
				sign = -sign;
			}

			DeleteMatrix(temp);
		}

		return D;
	}


	public static double Determinant(Matrix m){
		double D, n;

		n = NumberOfRows(m);
		D = DeterminantOfSubmatrix(m, n);

		return D;
	}


	public static void Adjoint(Matrix A, Matrix adj){
		double n, sign;
		Matrix cofactors;
		double i, j;

		n = A.r.Length;

		if(n == 1d){
			adj.r[0].c[0] = 1d;
		}else{
			cofactors = CreateSquareMatrix(n);

			for(i = 0d; i < n; i = i + 1d){
				for(j = 0d; j < n; j = j + 1d){
					CofactorOfMatrix(A, cofactors, i, j, n);

					if((i + j)%2d == 0d){
						sign = 1d;
					}else{
						sign = -1d;
					}

					adj.r[(int)(j)].c[(int)(i)] = sign*DeterminantOfSubmatrix(cofactors, n - 1d);
				}
			}

			DeleteMatrix(cofactors);
		}
	}


	public static bool Inverse(Matrix A, Matrix inverseResult){
		bool success;
		Matrix adj;
		double n, i, j;
		double det;

		if(NumberOfColumns(A) == NumberOfRows(A)){
			n = NumberOfColumns(A);

			det = Determinant(A);
			if(det != 0d){
				adj = CreateSquareMatrix(n);
				Adjoint(A, adj);

				for(i = 0d; i < n; i = i + 1d){
					for(j = 0d; j < n; j = j + 1d){
						inverseResult.r[(int)(i)].c[(int)(j)] = adj.r[(int)(i)].c[(int)(j)]/det;
					}
				}

				success = true;
				DeleteMatrix(adj);
			}else{
				success = false;
			}
		}else{
			success = false;
		}

		return success;
	}


	public static bool IsSymmetric(Matrix A){
		double N;
		double i, j;
		bool isx, done;

		N = NumberOfRows(A);

		done = false;
		isx = true;
		for(i = 0d; i < N && !done; i = i + 1d){
			for(j = 0d; j < i && !done; j = j + 1d){
				if(A.r[(int)(i)].c[(int)(j)] != A.r[(int)(j)].c[(int)(i)]){
					isx = false;
					done = true;
				}
			}
		}

		return isx;
	}


	public static bool IsSquare(Matrix A){
		bool isx;

		if(NumberOfRows(A) == NumberOfColumns(A)){
			isx = true;
		}else{
			isx = false;
		}

		return isx;
	}


	public static bool Cholesky(Matrix A, Matrix L){
		bool success;
		double N;
		double i, j, k, s;

		Clear(L);

		if(IsSquare(A) && IsSymmetric(A)){
			success = true;

			N = NumberOfRows(A);

			for(i = 0d; i < N && success; i = i + 1d){
				for(j = 0d; j <= i && success; j = j + 1d){
					s = 0d;
					for(k = 0d; k < j; k = k + 1d){
						s = s + L.r[(int)(i)].c[(int)(k)]*L.r[(int)(j)].c[(int)(k)];
					}
					if(i == j){
						L.r[(int)(i)].c[(int)(i)] = Sqrt(A.r[(int)(i)].c[(int)(i)] - s);
					}else{
						L.r[(int)(i)].c[(int)(j)] = 1d/L.r[(int)(j)].c[(int)(j)]*(A.r[(int)(i)].c[(int)(j)] - s);
					}
				}
				if(L.r[(int)(i)].c[(int)(i)] <= 0d){
					success = false;
				}
			}

			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static void Clear(Matrix a){
		Fill(a, 0d);
	}


	public static void Fill(Matrix a, double value){
		double m, n;

		for(m = 0d; m < NumberOfRows(a); m = m + 1d){
			for(n = 0d; n < NumberOfColumns(a); n = n + 1d){
				a.r[(int)(m)].c[(int)(n)] = value;
			}
		}
	}


	public static double Element(Matrix matrix, double m, double n){
		return matrix.r[(int)(m)].c[(int)(n)];
	}


	public static double Trace(Matrix a){
		double m;
		double d, tr;

		tr = 0d;

		d = a.r.Length;
		for(m = 0d; m < d; m = m + 1d){
			tr = tr + a.r[(int)(m)].c[(int)(m)];
		}

		return tr;
	}


	public static Matrix ColumnCombineMatricesToNew(Matrix A, Matrix B){
		Matrix X;
		double m, n;

		X = CreateMatrix(NumberOfRows(A), NumberOfColumns(A) + NumberOfColumns(B));

		for(m = 0d; m < NumberOfRows(A); m = m + 1d){
			for(n = 0d; n < NumberOfColumns(A); n = n + 1d){
				X.r[(int)(m)].c[(int)(n)] = A.r[(int)(m)].c[(int)(n)];
			}
		}

		for(m = 0d; m < NumberOfRows(B); m = m + 1d){
			for(n = 0d; n < NumberOfColumns(B); n = n + 1d){
				X.r[(int)(m)].c[(int)(NumberOfColumns(A) + n)] = B.r[(int)(m)].c[(int)(n)];
			}
		}

		return X;
	}


	public static double NumberOfRows(Matrix A){
		return A.r.Length;
	}


	public static double NumberOfColumns(Matrix A){
		return A.r[0].c.Length;
	}


	public static double [] CharacteristicPolynomial(Matrix A){
		double [] coeffs;
		Matrix dummy;

		dummy = CreateSquareMatrix(0d);

		coeffs = CharacteristicPolynomialWithInverse(A, false, dummy);

		DeleteMatrix(dummy);

		return coeffs;
	}


	public static double [] CharacteristicPolynomialWithInverse(Matrix A, bool returnInverse, Matrix AInverse){
		double [] coeffs;
		Matrix Mk, Mkm1, t1, I;
		double n, k;

		n = NumberOfRows(A);
		coeffs = new double [(int)(n + 1d)];
		coeffs[(int)(n)] = 1d;
		Mkm1 = CreateSquareMatrix(n);
		Fill(Mkm1, 0d);
		I = CreateIdentityMatrix(n);
		Mk = CreateSquareMatrix(n);
		t1 = CreateSquareMatrix(n);

		for(k = 1d; k <= n; k = k + 1d){
			/* M_k = A * M_(k-1) + c_(n-k+1) * I*/
			Multiply(Mk, A, Mkm1);
			Assign(t1, I);
			ScalarMultiply(t1, coeffs[(int)(n - k + 1d)]);
			Add(Mk, t1);

			/* c_(n-k) = -1/k * trace(A * M_k)*/
			Multiply(t1, A, Mk);
			coeffs[(int)(n - k)] = -1d/k*Trace(t1);

			/* done*/
			Assign(Mkm1, Mk);

			if(k == n && returnInverse){
				Assign(AInverse, Mk);
				ScalarDivide(AInverse, -coeffs[0]);
			}
		}

		DeleteMatrix(Mkm1);
		DeleteMatrix(I);
		DeleteMatrix(Mk);
		DeleteMatrix(t1);

		return coeffs;
	}


	public static Matrix InverseAlt2(Matrix A){
		Matrix inverse;
		double [] coeffs;

		inverse = CreateSquareMatrix(NumberOfRows(A));
		coeffs = CharacteristicPolynomialWithInverse(A, true, inverse);
		delete(coeffs);

		return inverse;
	}


	public static bool Eigenvalues(Matrix A, NumberArrayReference eigenValuesReference){
		double [] p;
		bool success;

		p = CharacteristicPolynomial(A);
		success = FindRoots(p, eigenValuesReference);

		return success;
	}


	public static void GaussianElimination(Matrix A){
		double h, k, m, n, maxElement, i, j, max, maxCandidate, f;

		m = NumberOfRows(A);
		n = NumberOfColumns(A);

		h = 0d;
		k = 0d;
		for(; h < m && k < n; ){
			maxElement = h;
			max = 0d;
			for(i = h; i < m; i = i + 1d){
				maxCandidate = Abs(Element(A, i, k));
				if(max < maxCandidate){
					maxElement = i;
					max = maxCandidate;
				}
			}
			if(A.r[(int)(maxElement)].c[(int)(k)] == 0d){
				k = k + 1d;
			}else{
				SwapRows(A, h, maxElement);
				for(i = h + 1d; i < m; i = i + 1d){
					f = Element(A, i, k)/Element(A, h, k);
					A.r[(int)(i)].c[(int)(k)] = 0d;
					for(j = k + 1d; j < n; j = j + 1d){
						A.r[(int)(i)].c[(int)(j)] = Element(A, i, j) - Element(A, h, j)*f;
					}
				}
				h = h + 1d;
				k = k + 1d;
			}
		}
	}


	public static Matrix GaussianEliminationToNew(Matrix A){
		Matrix X;

		X = CreateCopyOfMatrix(A);
		GaussianElimination(X);

		return X;
	}


	public static Matrix CreateCopyOfMatrix(Matrix A){
		Matrix X;

		X = CreateMatrix(NumberOfRows(A), NumberOfColumns(A));
		Assign(X, A);

		return X;
	}


	public static void SwapRows(Matrix A, double to, double from){
		double n;
		double c, t;

		c = NumberOfRows(A);
		for(n = 0d; n < c; n = n + 1d){
			t = A.r[(int)(to)].c[(int)(n)];
			A.r[(int)(to)].c[(int)(n)] = A.r[(int)(from)].c[(int)(n)];
			A.r[(int)(from)].c[(int)(n)] = t;
		}
	}


	public static bool Eigenvectors(Matrix A, MatrixArrayReference eigenVectorsReference){
		Matrix Id, t1, B, v;
		Matrix [] eigenVectorsResult;
		bool success;
		NumberArrayReference eigenValuesReference;
		double i, lambda, j, N, x, k;
		double [] ev;

		N = NumberOfRows(A);

		eigenValuesReference = new NumberArrayReference();
		success = Eigenvalues(A, eigenValuesReference);

		eigenVectorsResult = new Matrix [(int)(N)];

		if(success){
			ev = eigenValuesReference.numberArray;

			Id = CreateIdentityMatrix(N);
			t1 = CreateSquareMatrix(N);
			B = CreateSquareMatrix(N);

			for(j = 0d; j < ev.Length && success; j = j + 1d){
				lambda = ev[(int)(j)];

				/* B = A - lambda * Id*/
				Assign(t1, Id);
				ScalarMultiply(t1, lambda);

				Assign(B, A);
				Subtract(B, t1);

				GaussianElimination(B);

				v = CreateMatrix(N, 1d);
				v.r[(int)(N - 1d)].c[0] = 1d;
				for(i = N - 2d; i >= 0d && success; i = i - 1d){
					if(!RowIsZero(B, i)){
						x = 0d;

						for(k = N - 1d; k > i; k = k - 1d){
							x = x - Element(B, i, k)*Element(v, k, 0d);
						}

						v.r[(int)(i)].c[0] = x/Element(B, i, i);
					}else{
						success = false;
					}
				}

				eigenVectorsResult[(int)(j)] = v;
			}

			DeleteMatrix(t1);
			DeleteMatrix(B);
			DeleteMatrix(Id);

			eigenVectorsReference.matrices = eigenVectorsResult;
		}

		return success;
	}


	public static bool RowIsZero(Matrix X, double r){
		bool isZero;
		double columns, i;

		isZero = true;

		columns = NumberOfColumns(X);
		for(i = 0d; i < columns && isZero; i = i + 1d){
			if(Element(X, r, i) != 0d){
				isZero = false;
			}
		}

		return isZero;
	}


	public static void DeleteMatrix(Matrix X){
		double m, rows;

		rows = NumberOfRows(X);
		for(m = 0d; m < rows; m = m + 1d){
			delete(X.r[(int)(m)]);
		}

		delete(X.r);
		delete(X);
	}


	public static Matrix CreateDiagonalMatrixFromArray(double [] array){
		double m;
		Matrix matrix;

		matrix = CreateSquareMatrix(array.Length);
		Fill(matrix, 0d);

		for(m = 0d; m < array.Length; m = m + 1d){
			matrix.r[(int)(m)].c[(int)(m)] = array[(int)(m)];
		}

		return matrix;
	}


	public static Matrix CreateMatrixFromRowCopies(double [] row, double times){
		double m, n;
		Matrix matrix;

		matrix = CreateMatrix(times, row.Length);

		for(m = 0d; m < times; m = m + 1d){
			for(n = 0d; n < row.Length; n = n + 1d){
				matrix.r[(int)(m)].c[(int)(n)] = row[(int)(n)];
			}
		}

		return matrix;
	}


	public static double [] ExtractDiagonal(Matrix X){
		double [] diag;
		double n, i;

		n = NumberOfRows(X);
		diag = new double [(int)(n)];

		for(i = 0d; i < n; i = i + 1d){
			diag[(int)(i)] = X.r[(int)(i)].c[(int)(i)];
		}

		return diag;
	}


	public static bool MatrixEqualsEpsilon(Matrix b, Matrix f, double epsilon){
		double x, y, columns, rows;
		bool equals;

		equals = true;

		if(NumberOfRows(b) == NumberOfColumns(f) && NumberOfColumns(b) == NumberOfColumns(f)){
			columns = NumberOfColumns(b);
			rows = NumberOfRows(b);

			for(x = 0d; x < columns; x = x + 1d){
				for(y = 0d; y < rows; y = y + 1d){
					equals = equals && EpsilonCompare(b.r[(int)(x)].c[(int)(y)], f.r[(int)(x)].c[(int)(y)], epsilon);
				}
			}
		}else{
			equals = false;
		}

		return equals;
	}


	public static void AssertFalse(bool b, NumberReference failures){
		if(b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertTrue(bool b, NumberReference failures){
		if(!b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertEquals(double a, double b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertBooleansEqual(bool a, bool b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertCharactersEqual(char a, char b, NumberReference failures){
		if(a != b){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertStringEquals(char [] a, char [] b, NumberReference failures){
		if(!StringsEqual(a, b)){
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertNumberArraysEqual(double [] a, double [] b, NumberReference failures){
		double i;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertEquals(a[(int)(i)], b[(int)(i)], failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertBooleanArraysEqual(bool [] a, bool [] b, NumberReference failures){
		double i;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertBooleansEqual(a[(int)(i)], b[(int)(i)], failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static void AssertStringArraysEqual(StringReference [] a, StringReference [] b, NumberReference failures){
		double i;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length; i = i + 1d){
				AssertStringEquals(a[(int)(i)].stringx, b[(int)(i)].stringx, failures);
			}
		}else{
			failures.numberValue = failures.numberValue + 1d;
		}
	}


	public static bool FindRoots(double [] p, NumberArrayReference rootsReference){
		return LaguerresMethodWithRepeatedDivision(p, 100d, 0.000001, 0d, rootsReference);
	}


	public static bool LaguerresMethodWithRepeatedDivision(double [] p, double maxIterations, double precision, double guess, NumberArrayReference rootsReference){
		double n, nr, xk;
		double [] x;
		double [] q, r, d;
		bool success;
		NumberReference xkReference;
		n = pDegree(p);

		x = new double [(int)(n)];

		q = pCreatePolynomial(n);
		r = pCreatePolynomial(n);
		d = pCreatePolynomial(n);

		success = true;
		xkReference = CreateNumberReference(0d);

		for(nr = 0d; nr < n && success; nr = nr + 1d){
			success = LaguerresMethod(p, guess, maxIterations, precision, xkReference);
			if(success){
				xk = xkReference.numberValue;
				x[(int)(nr)] = xk;
				pFill(d, 0d);
				d[0] = -xk;
				d[1] = 1d;
				pDivide(q, r, p, d);
				pAssign(p, q);
			}
		}

		delete(q);
		delete(r);
		delete(d);
        
		rootsReference.numberArray = x;

		return success;
	}


	public static bool LaguerresMethod(double [] p, double guess, double maxIterations, double precision, NumberReference rootReference){
		double k, a, G, H, xk, denom1, denom2, denom, t1, n;
		bool success;
		n = pDegree(p);
		success = true;

		xk = guess;

		for(k = 0d; (k < maxIterations) && (Abs(pEvaluate(p, xk)) >= precision) && success; k = k + 1d){
			G = pEvaluateDerivative(p, xk, 1d)/pEvaluate(p, xk);
			H = Pow(G, 2d) - pEvaluateDerivative(p, xk, 2d)/pEvaluate(p, xk);
			t1 = (n - 1d)*(n*H - Pow(G, 2d));
			if(t1 >= 0d){
				denom = Sqrt(t1);
				denom1 = G + denom;
				denom2 = G - denom;
				if(Abs(denom1) >= Abs(denom2)){
					denom = denom1;
				}else{
					denom = denom2;
				}
				a = n/denom;
				xk = xk - a;
			}else{
				success = false;
			}
		}

		if(k == maxIterations){
			success = false;
		}

		if(Abs(pEvaluate(p, xk)) >= precision){
			success = false;
		}

		rootReference.numberValue = xk;

		return success;
	}


	public static BooleanReference CreateBooleanReference(bool value){
		BooleanReference refx;
		refx = new BooleanReference();
		refx.booleanValue = value;

		return refx;
	}


	public static BooleanArrayReference CreateBooleanArrayReference(bool [] value){
		BooleanArrayReference refx;
		refx = new BooleanArrayReference();
		refx.booleanArray = value;

		return refx;
	}


	public static BooleanArrayReference CreateBooleanArrayReferenceLengthValue(double length, bool value){
		BooleanArrayReference refx;
		double i;
		refx = new BooleanArrayReference();
		refx.booleanArray = new bool [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.booleanArray[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeBooleanArrayReference(BooleanArrayReference booleanArrayReference){
		delete(booleanArrayReference.booleanArray);
		delete(booleanArrayReference);
	}


	public static CharacterReference CreateCharacterReference(char value){
		CharacterReference refx;
		refx = new CharacterReference();
		refx.characterValue = value;

		return refx;
	}


	public static NumberReference CreateNumberReference(double value){
		NumberReference refx;
		refx = new NumberReference();
		refx.numberValue = value;

		return refx;
	}


	public static NumberArrayReference CreateNumberArrayReference(double [] value){
		NumberArrayReference refx;
		refx = new NumberArrayReference();
		refx.numberArray = value;

		return refx;
	}


	public static NumberArrayReference CreateNumberArrayReferenceLengthValue(double length, double value){
		NumberArrayReference refx;
		double i;
		refx = new NumberArrayReference();
		refx.numberArray = new double [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.numberArray[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeNumberArrayReference(NumberArrayReference numberArrayReference){
		delete(numberArrayReference.numberArray);
		delete(numberArrayReference);
	}


	public static StringReference CreateStringReference(char [] value){
		StringReference refx;
		refx = new StringReference();
		refx.stringx = value;

		return refx;
	}


	public static StringReference CreateStringReferenceLengthValue(double length, char value){
		StringReference refx;
		double i;
		refx = new StringReference();
		refx.stringx = new char [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.stringx[(int)(i)] = value;
		}

		return refx;
	}


	public static void FreeStringReference(StringReference stringReference){
		delete(stringReference.stringx);
		delete(stringReference);
	}


	public static StringArrayReference CreateStringArrayReference(StringReference [] strings){
		StringArrayReference refx;
		refx = new StringArrayReference();
		refx.stringArray = strings;

		return refx;
	}


	public static StringArrayReference CreateStringArrayReferenceLengthValue(double length, char [] value){
		StringArrayReference refx;
		double i;
		refx = new StringArrayReference();
		refx.stringArray = new StringReference [(int)(length)];

		for(i = 0d; i < length; i = i + 1d){
			refx.stringArray[(int)(i)] = CreateStringReference(value);
		}

		return refx;
	}


	public static void FreeStringArrayReference(StringArrayReference stringArrayReference){
		double i;
		for(i = 0d; i < stringArrayReference.stringArray.Length; i = i + 1d){
			delete(stringArrayReference.stringArray[(int)(i)]);
		}
		delete(stringArrayReference.stringArray);
		delete(stringArrayReference);
	}


	public static double Negate(double x){
		return -x;
	}


	public static double Positive(double x){
		return +x;
	}


	public static double Factorial(double x){
		double i, f;
		f = 1d;

		for(i = 2d; i <= x; i = i + 1d){
			f = f*i;
		}

		return f;
	}


	public static double Roundx(double x){
		return Floor(x + 0.5);
	}


	public static double BankersRound(double x){
		double r;
		if(Absolute(x - Truncate(x)) == 0.5){
			if(!DivisibleBy(Roundx(x), 2d)){
				r = Roundx(x) - 1d;
			}else{
				r = Roundx(x);
			}
		}else{
			r = Roundx(x);
		}

		return r;
	}


	public static double Ceil(double x){
		return Ceiling(x);
	}


	public static double Floorx(double x){
		return Floor(x);
	}


	public static double Truncate(double x){
		double t;
		if(x >= 0d){
			t = Floor(x);
		}else{
			t = Ceiling(x);
		}

		return t;
	}


	public static double Absolute(double x){
		return Abs(x);
	}


	public static double Logarithm(double x){
		return Log10(x);
	}


	public static double NaturalLogarithm(double x){
		return Log(x);
	}


	public static double Sinx(double x){
		return Sin(x);
	}


	public static double Cosx(double x){
		return Cos(x);
	}


	public static double Tanx(double x){
		return Tan(x);
	}


	public static double Asinx(double x){
		return Asin(x);
	}


	public static double Acosx(double x){
		return Acos(x);
	}


	public static double Atanx(double x){
		return Atan(x);
	}


	public static double Atan2(double y, double x){
		double a;
		a = 0d;

		if(x > 0d){
			a = Atanx(y/x);
		}else if(x < 0d && y >= 0d){
			a = Atanx(y/x) + PI;
		}else if(x < 0d && y < 0d){
			a = Atanx(y/x) - PI;
		}else if(x == 0d && y > 0d){
			a = PI/2d;
		}else if(x == 0d && y < 0d){
			a = -PI/2d;
		}

		return a;
	}


	public static double Squareroot(double x){
		return Sqrt(x);
	}


	public static double Expx(double x){
		return Exp(x);
	}


	public static bool DivisibleBy(double a, double b){
		return ((a%b) == 0d);
	}


	public static double Combinations(double n, double k){
		return Factorial(n)/(Factorial(n - k)*Factorial(k));
	}


	public static bool EpsilonCompareApproximateDigits(double a, double b, double digits){
		double ad, bd, d, epsilon;
		bool ret;
		if(a < 0d && b < 0d || a > 0d && b > 0d){
			if(a < 0d && b < 0d){
				a = -a;
				b = -b;
			}
			ad = Log10(a);
			bd = Log10(b);
			d = Max(ad, bd);
			epsilon = Pow(10d, d - digits);
			ret = Abs(a - b) > epsilon;
		}else{
			ret = false;
		}

		return ret;
	}


	public static bool EpsilonCompare(double a, double b, double epsilon){
		return Abs(a - b) < epsilon;
	}


	public static double GreatestCommonDivisor(double a, double b){
		double t;
		for(; b != 0d; ){
			t = b;
			b = a%b;
			a = t;
		}

		return a;
	}


	public static bool IsInteger(double a){
		return (a - Floor(a)) == 0d;
	}


	public static bool GreatestCommonDivisorWithCheck(double a, double b, NumberReference gcdReference){
		bool success;
		double gcd;
		if(IsInteger(a) && IsInteger(b)){
			gcd = GreatestCommonDivisor(a, b);
			gcdReference.numberValue = gcd;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static double LeastCommonMultiple(double a, double b){
		double lcm;
		if(a > 0d && b > 0d){
			lcm = Abs(a*b)/GreatestCommonDivisor(a, b);
		}else{
			lcm = 0d;
		}

		return lcm;
	}


	public static double Sign(double a){
		double s;
		if(a > 0d){
			s = 1d;
		}else if(a < 0d){
			s = -1d;
		}else{
			s = 0d;
		}

		return s;
	}


	public static double Maxx(double a, double b){
		return Max(a, b);
	}


	public static double Minx(double a, double b){
		return Min(a, b);
	}


	public static double Power(double a, double b){
		return Pow(a, b);
	}


	public static double [] StringToNumberArray(char [] stringx){
		double i;
		double [] array;
		array = new double [(int)(stringx.Length)];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			array[(int)(i)] = stringx[(int)(i)];
		}
		return array;
	}


	public static char [] NumberArrayToString(double [] array){
		double i;
		char [] stringx;
		stringx = new char [(int)(array.Length)];

		for(i = 0d; i < array.Length; i = i + 1d){
			stringx[(int)(i)] = (char)(array[(int)(i)]);
		}
		return stringx;
	}


	public static bool NumberArraysEqual(double [] a, double [] b){
		bool equal;
		double i;
		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static bool BooleanArraysEqual(bool [] a, bool [] b){
		bool equal;
		double i;
		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static bool StringsEqual(char [] a, char [] b){
		bool equal;
		double i;
		equal = true;
		if(a.Length == b.Length){
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(a[(int)(i)] != b[(int)(i)]){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static void FillNumberArray(double [] a, double value){
		double i;
		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static void FillString(char [] a, char value){
		double i;
		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static void FillBooleanArray(bool [] a, bool value){
		double i;
		for(i = 0d; i < a.Length; i = i + 1d){
			a[(int)(i)] = value;
		}
	}


	public static bool FillNumberArrayInterval(double [] a, double value, double from, double to){
		double i;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length){
			for(i = from; i < to; i = i + 1d){
				a[(int)(i)] = value;
			}
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool FillBooleanArrayInterval(bool [] a, bool value, double from, double to){
		double i;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length){
			for(i = Max(from, 0d); i < Min(to, a.Length); i = i + 1d){
				a[(int)(i)] = value;
			}
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool FillStringInterval(char [] a, char value, double from, double to){
		double i;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length){
			for(i = Max(from, 0d); i < Min(to, a.Length); i = i + 1d){
				a[(int)(i)] = value;
			}
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static double [] CopyNumberArray(double [] a){
		double i;
		double [] n;
		n = new double [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static bool [] CopyBooleanArray(bool [] a){
		double i;
		bool [] n;
		n = new bool [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static char [] CopyString(char [] a){
		double i;
		char [] n;
		n = new char [(int)(a.Length)];

		for(i = 0d; i < a.Length; i = i + 1d){
			n[(int)(i)] = a[(int)(i)];
		}

		return n;
	}


	public static bool CopyNumberArrayRange(double [] a, double from, double to, NumberArrayReference copyReference){
		double i, length;
		double [] n;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length && from <= to){
			length = to - from;
			n = new double [(int)(length)];
			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}
			copyReference.numberArray = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool CopyBooleanArrayRange(bool [] a, double from, double to, BooleanArrayReference copyReference){
		double i, length;
		bool [] n;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length && from <= to){
			length = to - from;
			n = new bool [(int)(length)];
			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}
			copyReference.booleanArray = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool CopyStringRange(char [] a, double from, double to, StringReference copyReference){
		double i, length;
		char [] n;
		bool success;
		if(from >= 0d && from < a.Length && to >= 0d && to < a.Length && from <= to){
			length = to - from;
			n = new char [(int)(length)];
			for(i = 0d; i < length; i = i + 1d){
				n[(int)(i)] = a[(int)(from + i)];
			}
			copyReference.stringx = n;
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static bool IsLastElement(double length, double index){
		return index + 1d == length;
	}


	public static double [] CreateNumberArray(double length, double value){
		double [] array;
		array = new double [(int)(length)];
		FillNumberArray(array, value);

		return array;
	}


	public static bool [] CreateBooleanArray(double length, bool value){
		bool [] array;
		array = new bool [(int)(length)];
		FillBooleanArray(array, value);

		return array;
	}


	public static char [] CreateString(double length, char value){
		char [] array;
		array = new char [(int)(length)];
		FillString(array, value);

		return array;
	}


	public static void pAdd(double [] a, double [] b){
		double i, nr;
		nr = b.Length;

		for(i = 0d; i < nr; i = i + 1d){
			a[(int)(i)] = a[(int)(i)] + b[(int)(i)];
		}
	}


	public static void pSubtract(double [] a, double [] b){
		double i, nr;
		nr = b.Length;

		for(i = 0d; i < nr; i = i + 1d){
			a[(int)(i)] = a[(int)(i)] - b[(int)(i)];
		}
	}


	public static void pMultiply(double [] c, double [] a, double [] b){
		double k, n, m, i;
		double av, bv;
		n = pDegree(a);
		m = pDegree(b);

		pFill(c, 0d);

		for(i = 0d; i <= n + m; i = i + 1d){
			c[(int)(i)] = 0d;
			for(k = 0d; k <= i && k < a.Length && i - k < b.Length; k = k + 1d){
				av = a[(int)(k)];
				bv = b[(int)(i - k)];
				c[(int)(i)] = c[(int)(i)] + av*bv;
			}
		}
	}


	public static void pDivide(double [] q, double [] r, double [] n, double [] d){
		double [] t, t1;
		double tcoff, tdegree, i;
		double deg;
		double rd, dd;
		pFill(q, 0d);
		pAssign(r, n);
		deg = pDegree(n);
		t = pCreatePolynomial(deg);
		t1 = pCreatePolynomial(deg);

		rd = pDegree(r);
		dd = pDegree(d);
		for(i = 0d; i < deg + 1d && !pIsZero(r) && rd - i >= dd; i = i + 1d){
			pFill(t, 0d);
			tdegree = rd - i - dd;
			tcoff = r[(int)(rd - i)]/d[(int)(dd)];
			t[(int)(tdegree)] = tcoff;
			pAdd(q, t);
			pFill(t1, 0d);
			pMultiply(t1, t, d);
			pSubtract(r, t1);
		}

		delete(t);
		delete(t1);
	}


	public static bool pIsZero(double [] a){
		double i;
		bool itIsZero;
		itIsZero = true;

		for(i = 0d; i < a.Length; i = i + 1d){
			if(a[(int)(i)] != 0d){
				itIsZero = false;
			}
		}

		return itIsZero;
	}


	public static void pAssign(double [] a, double [] b){
		double i, nr;
		nr = b.Length;

		for(i = 0d; i < nr; i = i + 1d){
			a[(int)(i)] = b[(int)(i)];
		}
	}


	public static double [] pCreatePolynomial(double deg){
		double [] p;
		p = new double [(int)(deg + 1d)];

		pFill(p, 0d);

		return p;
	}


	public static void pFill(double [] p, double value){
		double i;
		for(i = 0d; i < p.Length; i = i + 1d){
			p[(int)(i)] = value;
		}
	}


	public static double pDegree(double [] A){
		double i;
		double deg;
		bool done;
		done = false;
		deg = 0d;
		for(i = A.Length - 1d; i >= 0d && !done; i = i - 1d){
			if(A[(int)(i)] != 0d){
				deg = i;
				done = true;
			}
		}

		return deg;
	}


	public static double pLead(double [] A){
		double deg;
		deg = pDegree(A);

		return A[(int)(deg)];
	}


	public static double pEvaluate(double [] A, double x){
		double r, i;
		r = 0d;

		for(i = 0d; i < A.Length; i = i + 1d){
			r = r + A[(int)(i)]*Pow(x, i);
		}

		return r;
	}


	public static double pEvaluateDerivative(double [] A, double x, double n){
		double r, i, v;
		r = 0d;

		for(i = 0d; i < A.Length; i = i + 1d){
			if(i - n >= 0d){
				v = A[(int)(i)]*Factorial(n)*Combinations(i, n)*Pow(x, i - n);
				r = r + v;
			}
		}

		return r;
	}


	public static char [] nCreateStringDecimalFromNumber(double decimalx){
		StringReference stringReference;
		stringReference = new StringReference();

		/* This will succeed because base = 10.*/
		nCreateStringFromNumberWithCheck(decimalx, 10d, stringReference);

		return stringReference.stringx;
	}


	public static bool nCreateStringFromNumberWithCheck(double decimalx, double basex, StringReference stringReference){
		char [] stringx;
		double maximumDigits;
		double digitPosition;
		bool hasPrintedPoint;
		double i, d;
		bool success;
		CharacterReference characterReference;
		char c;
		if(decimalx == 0d){
			stringReference.stringx = "0".ToCharArray();
			success = true;
		}else{
			characterReference = new CharacterReference();
			if(IsInteger(basex)){
				success = true;
				stringx = new char [0];
				maximumDigits = nGetMaximumDigitsForBase(basex);
				digitPosition = nGetFirstDigitPosition(decimalx, basex);
				decimalx = (double)Round(decimalx*Pow(basex, maximumDigits - digitPosition - 1d));
				hasPrintedPoint = false;
				if(digitPosition < 0d){
					stringx = AppendCharacter(stringx, '0');
					stringx = AppendCharacter(stringx, '.');
					hasPrintedPoint = true;
					for(i = 0d; i < -digitPosition - 1d; i = i + 1d){
						stringx = AppendCharacter(stringx, '0');
					}
				}
				for(i = 0d; i < maximumDigits && success; i = i + 1d){
					d = Floor(decimalx/Pow(basex, maximumDigits - i - 1d));
					if(!hasPrintedPoint && digitPosition - i + 1d == 0d){
						if(decimalx != 0d){
							stringx = AppendCharacter(stringx, '.');
						}
						hasPrintedPoint = true;
					}
					if(decimalx == 0d && hasPrintedPoint){
					}else{
						success = nGetSingleDigitCharacterFromNumberWithCheck(d, basex, characterReference);
						if(success){
							c = characterReference.characterValue;
							stringx = AppendCharacter(stringx, c);
						}
					}
					if(success){
						decimalx = decimalx - d*Pow(basex, maximumDigits - i - 1d);
					}
				}
				if(success){
					for(i = 0d; i < digitPosition - maximumDigits + 1d; i = i + 1d){
						stringx = AppendCharacter(stringx, '0');
					}
					stringReference.stringx = stringx;
				}
			}else{
				success = false;
			}
		}

		/* Done*/
		return success;
	}


	public static double nGetMaximumDigitsForBase(double basex){
		double t;
		t = Pow(10d, 15d);
		return Floor(Log10(t)/Log10(basex));
	}


	public static double nGetFirstDigitPosition(double decimalx, double basex){
		double power;
		double t;
		power = Ceiling(Log10(decimalx)/Log10(basex));

		t = decimalx*Pow(basex, -power);
		if(t < basex && t >= 1d){
		}else if(t >= basex){
			power = power + 1d;
		}else if(t < 1d){
			power = power - 1d;
		}

		return power;
	}


	public static bool nGetSingleDigitCharacterFromNumberWithCheck(double c, double basex, CharacterReference characterReference){
		char [] numberTable;
		bool success;
		numberTable = nGetDigitCharacterTable();

		if(c < basex || c < numberTable.Length){
			success = true;
			characterReference.characterValue = numberTable[(int)(c)];
		}else{
			success = false;
		}

		return success;
	}


	public static char [] nGetDigitCharacterTable(){
		char [] numberTable;
		numberTable = new char [36];

		numberTable[0] = '0';
		numberTable[1] = '1';
		numberTable[2] = '2';
		numberTable[3] = '3';
		numberTable[4] = '4';
		numberTable[5] = '5';
		numberTable[6] = '6';
		numberTable[7] = '7';
		numberTable[8] = '8';
		numberTable[9] = '9';
		numberTable[10] = 'A';
		numberTable[11] = 'B';
		numberTable[12] = 'C';
		numberTable[13] = 'D';
		numberTable[14] = 'E';
		numberTable[15] = 'F';
		numberTable[16] = 'G';
		numberTable[17] = 'H';
		numberTable[18] = 'I';
		numberTable[19] = 'J';
		numberTable[20] = 'K';
		numberTable[21] = 'L';
		numberTable[22] = 'M';
		numberTable[23] = 'N';
		numberTable[24] = 'O';
		numberTable[25] = 'P';
		numberTable[26] = 'Q';
		numberTable[27] = 'R';
		numberTable[28] = 'S';
		numberTable[29] = 'T';
		numberTable[30] = 'U';
		numberTable[31] = 'V';
		numberTable[32] = 'W';
		numberTable[33] = 'X';
		numberTable[34] = 'Y';
		numberTable[35] = 'Z';

		return numberTable;
	}


	public static bool nCreateNumberFromDecimalStringWithCheck(char [] stringx, NumberReference decimalReference, StringReference errorMessage){
		return nCreateNumberFromStringWithCheck(stringx, 10d, decimalReference, errorMessage);
	}


	public static double nCreateNumberFromDecimalString(char [] stringx){
		NumberReference doubleReference;
		StringReference stringReference;
		double number;
		doubleReference = CreateNumberReference(0d);
		stringReference = CreateStringReference("".ToCharArray());
		nCreateNumberFromStringWithCheck(stringx, 10d, doubleReference, stringReference);
		number = doubleReference.numberValue;

		delete(doubleReference);
		delete(stringReference);

		return number;
	}


	public static bool nCreateNumberFromStringWithCheck(char [] stringx, double basex, NumberReference decimalReference, StringReference errorMessage){
		char c;
		double i, j;
		bool isPositive;
		double beforeDecimalPoint, afterDecimalPoint;
		double n, validCharacters, d;
		bool success;
		success = true;
		i = 0d;
		isPositive = true;
		beforeDecimalPoint = 0d;
		afterDecimalPoint = 0d;
		n = 0d;
		validCharacters = 0d;

		if(basex >= 2d && basex <= 36d){
			for(j = 0d; j < stringx.Length; j = j + 1d){
				c = stringx[(int)(j)];
				if(nCharacterIsNumberCharacterInBase(c, basex) || c == '.' || c == '-'){
					validCharacters = validCharacters + 1d;
				}
			}
			if(validCharacters == stringx.Length){
				if(stringx.Length > 0d){
					c = stringx[(int)(i)];
					if(c == '-'){
						isPositive = false;
						i = i + 1d;
					}
					if(i < stringx.Length){
						c = stringx[(int)(i)];
						if(nCharacterIsNumberCharacterInBase(c, basex)){
							for(; nCharacterIsNumberCharacterInBase(c, basex) && (i < stringx.Length); ){
								beforeDecimalPoint = beforeDecimalPoint + 1d;
								i = i + 1d;
								if(i < stringx.Length){
									c = stringx[(int)(i)];
								}
							}
							if(i < stringx.Length){
								c = stringx[(int)(i)];
								if(c == '.'){
									i = i + 1d;
									if(i < stringx.Length){
										c = stringx[(int)(i)];
										for(; nCharacterIsNumberCharacterInBase(c, basex) && (i < stringx.Length); ){
											afterDecimalPoint = afterDecimalPoint + 1d;
											i = i + 1d;
											if(i < stringx.Length){
												c = stringx[(int)(i)];
											}
										}
									}else{
										success = false;
										errorMessage.stringx = "Number must have digits after the decimal point.".ToCharArray();
									}
								}
							}
						}else{
							success = false;
							errorMessage.stringx = "Number must start with digits (for negative numbers, after the negative sign).".ToCharArray();
						}
					}
					if(success != false){
						if(isPositive){
							i = 0d;
						}else{
							i = 1d;
						}
						for(j = 0d; j < beforeDecimalPoint; j = j + 1d){
							c = stringx[(int)(i)];
							i = i + 1d;
							d = nGetNumberFromNumberCharacterForBase(c, basex);
							n = n + d*Pow(basex, beforeDecimalPoint - j - 1d);
						}
						if(afterDecimalPoint > 0d){
							i = i + 1d;
							for(j = 0d; j < afterDecimalPoint; j = j + 1d){
								c = stringx[(int)(i)];
								i = i + 1d;
								d = nGetNumberFromNumberCharacterForBase(c, basex);
								n = n + d*Pow(basex, 0d - j - 1d);
							}
						}
						if(!isPositive){
							n = -n;
						}
						decimalReference.numberValue = n;
						success = true;
					}
				}else{
					success = false;
					errorMessage.stringx = "String has no content.".ToCharArray();
				}
			}else{
				success = false;
				errorMessage.stringx = "String contains invalid character.".ToCharArray();
			}
		}else{
			success = false;
			errorMessage.stringx = "Base must be from 2 to 36.".ToCharArray();
		}

		return success;
	}


	public static double nGetNumberFromNumberCharacterForBase(char c, double basex){
		char [] numberTable;
		double i;
		double position;
		numberTable = nGetDigitCharacterTable();
		position = 0d;

		for(i = 0d; i < basex; i = i + 1d){
			if(numberTable[(int)(i)] == c){
				position = i;
			}
		}

		return position;
	}


	public static bool nCharacterIsNumberCharacterInBase(char c, double basex){
		char [] numberTable;
		double i;
		bool found;
		numberTable = nGetDigitCharacterTable();
		found = false;

		for(i = 0d; i < basex; i = i + 1d){
			if(numberTable[(int)(i)] == c){
				found = true;
			}
		}

		return found;
	}


	public static double [] nStringToNumberArray(char [] str){
		NumberArrayReference numberArrayReference;
		StringReference stringReference;
		double [] numbers;
		numberArrayReference = new NumberArrayReference();
		stringReference = new StringReference();

		nStringToNumberArrayWithCheck(str, numberArrayReference, stringReference);

		numbers = numberArrayReference.numberArray;

		delete(numberArrayReference);
		delete(stringReference);

		return numbers;
	}


	public static bool nStringToNumberArrayWithCheck(char [] str, NumberArrayReference numberArrayReference, StringReference errorMessage){
		StringReference [] numberStrings;
		double [] numbers;
		double i;
		char [] numberString, trimmedNumberString;
		bool success;
		NumberReference numberReference;
		numberStrings = SplitByString(str, ",".ToCharArray());

		numbers = new double [(int)(numberStrings.Length)];
		success = true;
		numberReference = new NumberReference();

		for(i = 0d; i < numberStrings.Length; i = i + 1d){
			numberString = numberStrings[(int)(i)].stringx;
			trimmedNumberString = Trim(numberString);
			success = nCreateNumberFromDecimalStringWithCheck(trimmedNumberString, numberReference, errorMessage);
			numbers[(int)(i)] = numberReference.numberValue;
			FreeStringReference(numberStrings[(int)(i)]);
			delete(trimmedNumberString);
		}

		delete(numberStrings);
		delete(numberReference);

		numberArrayReference.numberArray = numbers;

		return success;
	}


	public static bool SubstringWithCheck(char [] stringx, double from, double to, StringReference stringReference){
		bool success;
		if(from < stringx.Length && to < stringx.Length && from <= to && from >= 0d && to >= 0d){
			stringReference.stringx = Substring(stringx, from, to);
			success = true;
		}else{
			success = false;
		}

		return success;
	}


	public static char [] Substring(char [] stringx, double from, double to){
		char [] n;
		double i;
		n = new char [(int)(Max(to - from, 0d))];

		for(i = from; i < to; i = i + 1d){
			n[(int)(i - from)] = stringx[(int)(i)];
		}

		return n;
	}


	public static char [] AppendString(char [] stringx, char [] s){
		char [] newString;
		newString = ConcatenateString(stringx, s);

		delete(stringx);

		return newString;
	}


	public static char [] ConcatenateString(char [] stringx, char [] s){
		char [] newString;
		double i;
		newString = new char [(int)(stringx.Length + s.Length)];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			newString[(int)(i)] = stringx[(int)(i)];
		}

		for(i = 0d; i < s.Length; i = i + 1d){
			newString[(int)(stringx.Length + i)] = s[(int)(i)];
		}

		return newString;
	}


	public static char [] AppendCharacter(char [] stringx, char c){
		char [] newString;
		newString = ConcatenateCharacter(stringx, c);

		delete(stringx);

		return newString;
	}


	public static char [] ConcatenateCharacter(char [] stringx, char c){
		char [] newString;
		double i;
		newString = new char [(int)(stringx.Length + 1d)];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			newString[(int)(i)] = stringx[(int)(i)];
		}

		newString[(int)(stringx.Length)] = c;

		return newString;
	}


	public static StringReference [] SplitByCharacter(char [] toSplit, char splitBy){
		StringReference [] split;
		char [] stringToSplitBy;
		stringToSplitBy = new char [1];
		stringToSplitBy[0] = splitBy;

		split = SplitByString(toSplit, stringToSplitBy);

		delete(stringToSplitBy);

		return split;
	}


	public static bool IndexOfCharacter(char [] stringx, char character, NumberReference indexReference){
		double i;
		bool found;
		found = false;
		for(i = 0d; i < stringx.Length && !found; i = i + 1d){
			if(stringx[(int)(i)] == character){
				found = true;
				indexReference.numberValue = i;
			}
		}

		return found;
	}


	public static bool SubstringEqualsWithCheck(char [] stringx, double from, char [] substring, BooleanReference equalsReference){
		bool success;
		if(from < stringx.Length){
			success = true;
			equalsReference.booleanValue = SubstringEquals(stringx, from, substring);
		}else{
			success = false;
		}

		return success;
	}


	public static bool SubstringEquals(char [] stringx, double from, char [] substring){
		double i;
		bool equal;
		equal = true;
		for(i = 0d; i < substring.Length && equal; i = i + 1d){
			if(stringx[(int)(from + i)] != substring[(int)(i)]){
				equal = false;
			}
		}

		return equal;
	}


	public static bool IndexOfString(char [] stringx, char [] substring, NumberReference indexReference){
		double i;
		bool found;
		found = false;
		for(i = 0d; i < stringx.Length - substring.Length + 1d && !found; i = i + 1d){
			if(SubstringEquals(stringx, i, substring)){
				found = true;
				indexReference.numberValue = i;
			}
		}

		return found;
	}


	public static bool ContainsCharacter(char [] stringx, char character){
		return IndexOfCharacter(stringx, character, new NumberReference());
	}


	public static bool ContainsString(char [] stringx, char [] substring){
		return IndexOfString(stringx, substring, new NumberReference());
	}


	public static void ToUpperCase(char [] stringx){
		double i;
		for(i = 0d; i < stringx.Length; i = i + 1d){
			stringx[(int)(i)] = charToUpperCase(stringx[(int)(i)]);
		}
	}


	public static void ToLowerCase(char [] stringx){
		double i;
		for(i = 0d; i < stringx.Length; i = i + 1d){
			stringx[(int)(i)] = charToLowerCase(stringx[(int)(i)]);
		}
	}


	public static bool EqualsIgnoreCase(char [] a, char [] b){
		bool equal;
		double i;
		if(a.Length == b.Length){
			equal = true;
			for(i = 0d; i < a.Length && equal; i = i + 1d){
				if(charToLowerCase(a[(int)(i)]) != charToLowerCase(b[(int)(i)])){
					equal = false;
				}
			}
		}else{
			equal = false;
		}

		return equal;
	}


	public static char [] ReplacesString(char [] stringx, char [] toReplace, char [] replaceWith){
		char [] result;
		double i;
		BooleanReference equalsReference;
		bool success;
		equalsReference = new BooleanReference();
		result = new char [0];

		for(i = 0d; i < stringx.Length; ){
			success = SubstringEqualsWithCheck(stringx, i, toReplace, equalsReference);
			if(success){
				success = equalsReference.booleanValue;
			}
			if(success && toReplace.Length > 0d){
				result = ConcatenateString(result, replaceWith);
				i = i + toReplace.Length;
			}else{
				result = ConcatenateCharacter(result, stringx[(int)(i)]);
				i = i + 1d;
			}
		}

		return result;
	}


	public static char [] ReplaceCharacter(char [] stringx, char toReplace, char replaceWith){
		char [] result;
		double i;
		result = new char [0];

		for(i = 0d; i < stringx.Length; i = i + 1d){
			if(stringx[(int)(i)] == toReplace){
				result = ConcatenateCharacter(result, replaceWith);
			}else{
				result = ConcatenateCharacter(result, stringx[(int)(i)]);
			}
		}

		return result;
	}


	public static char [] Trim(char [] stringx){
		char [] result;
		double i, lastWhitespaceLocationStart, lastWhitespaceLocationEnd;
		bool firstNonWhitespaceFound;
		lastWhitespaceLocationStart = -1d;
		firstNonWhitespaceFound = false;
		for(i = 0d; i < stringx.Length && !firstNonWhitespaceFound; i = i + 1d){
			if(charIsWhiteSpace(stringx[(int)(i)])){
				lastWhitespaceLocationStart = i;
			}else{
				firstNonWhitespaceFound = true;
			}
		}

		/* Find whitepaces at the end.*/
		lastWhitespaceLocationEnd = stringx.Length;
		firstNonWhitespaceFound = false;
		for(i = stringx.Length - 1d; i >= 0d && !firstNonWhitespaceFound; i = i - 1d){
			if(charIsWhiteSpace(stringx[(int)(i)])){
				lastWhitespaceLocationEnd = i;
			}else{
				firstNonWhitespaceFound = true;
			}
		}

		if(lastWhitespaceLocationStart < lastWhitespaceLocationEnd){
			result = Substring(stringx, lastWhitespaceLocationStart + 1d, lastWhitespaceLocationEnd);
		}else{
			result = new char [0];
		}

		return result;
	}


	public static bool StartsWith(char [] stringx, char [] start){
		bool startsWithString;
		startsWithString = false;
		if(stringx.Length >= start.Length){
			startsWithString = SubstringEquals(stringx, 0d, start);
		}

		return startsWithString;
	}


	public static bool EndsWith(char [] stringx, char [] end){
		bool endsWithString;
		endsWithString = false;
		if(stringx.Length >= end.Length){
			endsWithString = SubstringEquals(stringx, stringx.Length - end.Length, end);
		}

		return endsWithString;
	}


	public static StringReference [] SplitByString(char [] toSplit, char [] splitBy){
		StringReference [] split;
		char [] next;
		double i;
		char c;
		StringReference n;
		split = new StringReference [0];

		next = new char [0];
		for(i = 0d; i < toSplit.Length; ){
			c = toSplit[(int)(i)];
			if(SubstringEquals(toSplit, i, splitBy)){
				if(split.Length != 0d || i != 0d){
					n = new StringReference();
					n.stringx = next;
					split = AddString(split, n);
					next = new char [0];
					i = i + splitBy.Length;
				}
			}else{
				next = AppendCharacter(next, c);
				i = i + 1d;
			}
		}

		if(next.Length > 0d){
			n = new StringReference();
			n.stringx = next;
			split = AddString(split, n);
		}

		return split;
	}


	public static double [] AddNumber(double [] list, double a){
		double [] newlist;
		double i;
		newlist = new double [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddNumberRef(NumberArrayReference list, double i){
		list.numberArray = AddNumber(list.numberArray, i);
	}


	public static double [] RemoveNumber(double [] list, double n){
		double [] newlist;
		double i;
		newlist = new double [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}
		
		delete(list);
		
		return newlist;
	}


	public static double GetNumberRef(NumberArrayReference list, double i){
		return list.numberArray[(int)(i)];
	}


	public static void RemoveNumberRef(NumberArrayReference list, double i){
		list.numberArray = RemoveNumber(list.numberArray, i);
	}


	public static StringReference [] AddString(StringReference [] list, StringReference a){
		StringReference [] newlist;
		double i;
		newlist = new StringReference [(int)(list.Length + 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddStringRef(StringArrayReference list, StringReference i){
		list.stringArray = AddString(list.stringArray, i);
	}


	public static StringReference [] RemoveString(StringReference [] list, double n){
		StringReference [] newlist;
		double i;
		newlist = new StringReference [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}
		
		delete(list);
		
		return newlist;
	}


	public static StringReference GetStringRef(StringArrayReference list, double i){
		return list.stringArray[(int)(i)];
	}


	public static void RemoveStringRef(StringArrayReference list, double i){
		list.stringArray = RemoveString(list.stringArray, i);
	}


	public static bool [] AddBoolean(bool [] list, bool a){
		bool [] newlist;
		double i;
		newlist = new bool [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddBooleanRef(BooleanArrayReference list, bool i){
		list.booleanArray = AddBoolean(list.booleanArray, i);
	}


	public static bool [] RemoveBoolean(bool [] list, double n){
		bool [] newlist;
		double i;
		newlist = new bool [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}
		
		delete(list);
		
		return newlist;
	}


	public static bool GetBooleanRef(BooleanArrayReference list, double i){
		return list.booleanArray[(int)(i)];
	}


	public static void RemoveDecimalRef(BooleanArrayReference list, double i){
		list.booleanArray = RemoveBoolean(list.booleanArray, i);
	}


	public static char [] AddCharacter(char [] list, char a){
		char [] newlist;
		double i;
		newlist = new char [(int)(list.Length + 1d)];
		for(i = 0d; i < list.Length; i = i + 1d){
			newlist[(int)(i)] = list[(int)(i)];
		}
		newlist[(int)(list.Length)] = a;
		
		delete(list);
		
		return newlist;
	}


	public static void AddCharacterRef(StringReference list, char i){
		list.stringx = AddCharacter(list.stringx, i);
	}


	public static char [] RemoveCharacter(char [] list, double n){
		char [] newlist;
		double i;
		newlist = new char [(int)(list.Length - 1d)];

		for(i = 0d; i < list.Length; i = i + 1d){
			if(i < n){
				newlist[(int)(i)] = list[(int)(i)];
			}
			if(i > n){
				newlist[(int)(i - 1d)] = list[(int)(i)];
			}
		}

		delete(list);

		return newlist;
	}


	public static char GetCharacterRef(StringReference list, double i){
		return list.stringx[(int)(i)];
	}


	public static void RemoveCharacterRef(StringReference list, double i){
		list.stringx = RemoveCharacter(list.stringx, i);
	}


	public static char charToLowerCase(char character){
		char toReturn;
		toReturn = character;
		if(character == 'A'){
			toReturn = 'a';
		}else if(character == 'B'){
			toReturn = 'b';
		}else if(character == 'C'){
			toReturn = 'c';
		}else if(character == 'D'){
			toReturn = 'd';
		}else if(character == 'E'){
			toReturn = 'e';
		}else if(character == 'F'){
			toReturn = 'f';
		}else if(character == 'G'){
			toReturn = 'g';
		}else if(character == 'H'){
			toReturn = 'h';
		}else if(character == 'I'){
			toReturn = 'i';
		}else if(character == 'J'){
			toReturn = 'j';
		}else if(character == 'K'){
			toReturn = 'k';
		}else if(character == 'L'){
			toReturn = 'l';
		}else if(character == 'M'){
			toReturn = 'm';
		}else if(character == 'N'){
			toReturn = 'n';
		}else if(character == 'O'){
			toReturn = 'o';
		}else if(character == 'P'){
			toReturn = 'p';
		}else if(character == 'Q'){
			toReturn = 'q';
		}else if(character == 'R'){
			toReturn = 'r';
		}else if(character == 'S'){
			toReturn = 's';
		}else if(character == 'T'){
			toReturn = 't';
		}else if(character == 'U'){
			toReturn = 'u';
		}else if(character == 'V'){
			toReturn = 'v';
		}else if(character == 'W'){
			toReturn = 'w';
		}else if(character == 'X'){
			toReturn = 'x';
		}else if(character == 'Y'){
			toReturn = 'y';
		}else if(character == 'Z'){
			toReturn = 'z';
		}

		return toReturn;
	}


	public static char charToUpperCase(char character){
		char toReturn;
		toReturn = character;
		if(character == 'a'){
			toReturn = 'A';
		}else if(character == 'b'){
			toReturn = 'B';
		}else if(character == 'c'){
			toReturn = 'C';
		}else if(character == 'd'){
			toReturn = 'D';
		}else if(character == 'e'){
			toReturn = 'E';
		}else if(character == 'f'){
			toReturn = 'F';
		}else if(character == 'g'){
			toReturn = 'G';
		}else if(character == 'h'){
			toReturn = 'H';
		}else if(character == 'i'){
			toReturn = 'I';
		}else if(character == 'j'){
			toReturn = 'J';
		}else if(character == 'k'){
			toReturn = 'K';
		}else if(character == 'l'){
			toReturn = 'L';
		}else if(character == 'm'){
			toReturn = 'M';
		}else if(character == 'n'){
			toReturn = 'N';
		}else if(character == 'o'){
			toReturn = 'O';
		}else if(character == 'p'){
			toReturn = 'P';
		}else if(character == 'q'){
			toReturn = 'Q';
		}else if(character == 'r'){
			toReturn = 'R';
		}else if(character == 's'){
			toReturn = 'S';
		}else if(character == 't'){
			toReturn = 'T';
		}else if(character == 'u'){
			toReturn = 'U';
		}else if(character == 'v'){
			toReturn = 'V';
		}else if(character == 'w'){
			toReturn = 'W';
		}else if(character == 'x'){
			toReturn = 'X';
		}else if(character == 'y'){
			toReturn = 'Y';
		}else if(character == 'z'){
			toReturn = 'Z';
		}

		return toReturn;
	}


	public static bool charIsUpperCase(char character){
		bool isUpper;
		isUpper = false;
		if(character == 'A'){
			isUpper = true;
		}else if(character == 'B'){
			isUpper = true;
		}else if(character == 'C'){
			isUpper = true;
		}else if(character == 'D'){
			isUpper = true;
		}else if(character == 'E'){
			isUpper = true;
		}else if(character == 'F'){
			isUpper = true;
		}else if(character == 'G'){
			isUpper = true;
		}else if(character == 'H'){
			isUpper = true;
		}else if(character == 'I'){
			isUpper = true;
		}else if(character == 'J'){
			isUpper = true;
		}else if(character == 'K'){
			isUpper = true;
		}else if(character == 'L'){
			isUpper = true;
		}else if(character == 'M'){
			isUpper = true;
		}else if(character == 'N'){
			isUpper = true;
		}else if(character == 'O'){
			isUpper = true;
		}else if(character == 'P'){
			isUpper = true;
		}else if(character == 'Q'){
			isUpper = true;
		}else if(character == 'R'){
			isUpper = true;
		}else if(character == 'S'){
			isUpper = true;
		}else if(character == 'T'){
			isUpper = true;
		}else if(character == 'U'){
			isUpper = true;
		}else if(character == 'V'){
			isUpper = true;
		}else if(character == 'W'){
			isUpper = true;
		}else if(character == 'X'){
			isUpper = true;
		}else if(character == 'Y'){
			isUpper = true;
		}else if(character == 'Z'){
			isUpper = true;
		}

		return isUpper;
	}


	public static bool charIsLowerCase(char character){
		bool isLower;
		isLower = false;
		if(character == 'a'){
			isLower = true;
		}else if(character == 'b'){
			isLower = true;
		}else if(character == 'c'){
			isLower = true;
		}else if(character == 'd'){
			isLower = true;
		}else if(character == 'e'){
			isLower = true;
		}else if(character == 'f'){
			isLower = true;
		}else if(character == 'g'){
			isLower = true;
		}else if(character == 'h'){
			isLower = true;
		}else if(character == 'i'){
			isLower = true;
		}else if(character == 'j'){
			isLower = true;
		}else if(character == 'k'){
			isLower = true;
		}else if(character == 'l'){
			isLower = true;
		}else if(character == 'm'){
			isLower = true;
		}else if(character == 'n'){
			isLower = true;
		}else if(character == 'o'){
			isLower = true;
		}else if(character == 'p'){
			isLower = true;
		}else if(character == 'q'){
			isLower = true;
		}else if(character == 'r'){
			isLower = true;
		}else if(character == 's'){
			isLower = true;
		}else if(character == 't'){
			isLower = true;
		}else if(character == 'u'){
			isLower = true;
		}else if(character == 'v'){
			isLower = true;
		}else if(character == 'w'){
			isLower = true;
		}else if(character == 'x'){
			isLower = true;
		}else if(character == 'y'){
			isLower = true;
		}else if(character == 'z'){
			isLower = true;
		}

		return isLower;
	}


	public static bool charIsLetter(char character){
		return charIsUpperCase(character) || charIsLowerCase(character);
	}


	public static bool charIsNumber(char character){
		bool isNumber;
		isNumber = false;
		if(character == '0'){
			isNumber = true;
		}else if(character == '1'){
			isNumber = true;
		}else if(character == '2'){
			isNumber = true;
		}else if(character == '3'){
			isNumber = true;
		}else if(character == '4'){
			isNumber = true;
		}else if(character == '5'){
			isNumber = true;
		}else if(character == '6'){
			isNumber = true;
		}else if(character == '7'){
			isNumber = true;
		}else if(character == '8'){
			isNumber = true;
		}else if(character == '9'){
			isNumber = true;
		}

		return isNumber;
	}


	public static bool charIsWhiteSpace(char character){
		bool isWhiteSpace;
		isWhiteSpace = false;
		if(character == ' '){
			isWhiteSpace = true;
		}else if(character == '\t'){
			isWhiteSpace = true;
		}else if(character == '\n'){
			isWhiteSpace = true;
		}else if(character == '\r'){
			isWhiteSpace = true;
		}

		return isWhiteSpace;
	}


	public static bool charIsSymbol(char character){
		bool isSymbol;
		isSymbol = false;
		if(character == '!'){
			isSymbol = true;
		}else if(character == '\"'){
			isSymbol = true;
		}else if(character == '#'){
			isSymbol = true;
		}else if(character == '$'){
			isSymbol = true;
		}else if(character == '%'){
			isSymbol = true;
		}else if(character == '&'){
			isSymbol = true;
		}else if(character == '\''){
			isSymbol = true;
		}else if(character == '('){
			isSymbol = true;
		}else if(character == ')'){
			isSymbol = true;
		}else if(character == '*'){
			isSymbol = true;
		}else if(character == '+'){
			isSymbol = true;
		}else if(character == ','){
			isSymbol = true;
		}else if(character == '-'){
			isSymbol = true;
		}else if(character == '.'){
			isSymbol = true;
		}else if(character == '/'){
			isSymbol = true;
		}else if(character == ':'){
			isSymbol = true;
		}else if(character == ';'){
			isSymbol = true;
		}else if(character == '<'){
			isSymbol = true;
		}else if(character == '='){
			isSymbol = true;
		}else if(character == '>'){
			isSymbol = true;
		}else if(character == '?'){
			isSymbol = true;
		}else if(character == '@'){
			isSymbol = true;
		}else if(character == '['){
			isSymbol = true;
		}else if(character == '\\'){
			isSymbol = true;
		}else if(character == ']'){
			isSymbol = true;
		}else if(character == '^'){
			isSymbol = true;
		}else if(character == '_'){
			isSymbol = true;
		}else if(character == '`'){
			isSymbol = true;
		}else if(character == '{'){
			isSymbol = true;
		}else if(character == '|'){
			isSymbol = true;
		}else if(character == '}'){
			isSymbol = true;
		}else if(character == '~'){
			isSymbol = true;
		}

		return isSymbol;
	}


	public static void delete(System.Object objectx){
		// C# has garbage collection.
	}
}

}